
import logging
from unittest.mock import patch
from odoo.tools import convert_file
from odoo.tests import tagged, new_test_user
from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.util import version_gte, version_lte
from odoo.addons.saas_worker.tests.common import SaasWorkerHttpCommon


@tagged('post_install', '-at_install')
class TestTrialCommon(SaasWorkerHttpCommon):
    _existing_odoocom_users = set()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        if not metabase.get(cls.env.cr.dbname):
            cls.skipTest(cls, "database '%s' not registered in meta base." % cls.env.cr.dbname)

        ResPartner_class = type(cls.env['res.partner'])
        patcher = patch.object(ResPartner_class, '_user_exists_odoocom', autospec=True,
                               side_effect=lambda s: s.user_ids and s.user_ids.login in cls._existing_odoocom_users)
        patcher.start()
        cls.addClassCleanup(patcher.stop)

        cls.user_admin = cls.env.ref('base.user_admin')
        cls.user_employee = new_test_user(
            cls.env,
            login='employee@example.net', password='employee@example.net',
        )

        if version_gte("15.0"):
            class FilterDeprecatedTRawWarning(logging.Filter):
                def filter(self, record):
                    if """Found deprecated directive '@t-raw="rr_widget_content"'""" in record.getMessage():
                        # RollingRelease template do contains a deprecated t-raw directive, but
                        # it's only reacheable for odoo <15.0, so ignore this warning issued by
                        # qweb2.js (browser) during testing.
                        return False
                    return True
            cls._logger.getChild('browser').addFilter(FilterDeprecatedTRawWarning())

        # ensure user tours are disabled even for database created without demo
        # (ex. through TestTrialsCreation) as this can cause intermittent
        # tours failure if user tours are running concurrently of the test tour
        if 'web_tour' in cls.env.registry._init_modules:
            # For >=saas~14.3 setting the 'demo' flag on at least one module is
            # sufficient (see https://github.com/odoo/odoo/commit/e0efbb08a4f5)
            # but for older version (<=saas~14.2) we need to ensure that
            # "tour_disable.js" will be loaded in the assets.
            cls.env.ref("base.module_web_tour").demo = True
            if version_lte("saas~14.2"):
                convert_file(cls.env.cr, "web_tour", "data/web_tour_demo.xml", {})

    def start_tour(self, url_path, tour_name, step_delay=None, **kwargs):
        login = kwargs.get('login')
        if login:
            # on trial databases, trial_init_db() assigned a random password;
            # so as a safety we always reset user password before starting a
            # tour (for test we always asume password = login)
            user = self.env["res.users"].search([('login', '=', login)])
            user.sudo().with_context(saas_force_oauth=True, no_update_notification=True).write({
                'password': user.login
            })
        return super().start_tour(url_path, tour_name, step_delay=step_delay, **kwargs)
